//----------------------------------------------------
//
// File:   SampleDA.c
// Author: Patrice BERNARD (metro@nanika.net)
//
// Main source file for project SampleDA
// Directly converted from SampleDA (for PP Compiler) by Palmipod
// See http://www.ppcompiler.org/articles.php?lng=en&pg=446
//
// DA specs at http://www001.upp.so-net.ne.jp/h-yama/da/da-spec10-e.html
//
//----------------------------------------------------
//
// Version 1.0.0 (19/08/2006)
// - Creation
//
//----------------------------------------------------

#include <PalmOS.h>
#include <standalone.h>
#include "SampleDA.rh"

// A DA has a single code resource with ID 1000
STANDALONE_CODE_RESOURCE_ID(1000);

//----------------------------------------------------
// Manage events in main form
//----------------------------------------------------
static Boolean MainFormHandleEvent(EventType * event)
{
	FormPtr form;
	Boolean done = false;
	RectangleType rect;
	Coord x, y;

	switch (event->eType)
	{
		//----------------------------------------
		// Control selection event
		//----------------------------------------
		case ctlSelectEvent:
		form = FrmGetActiveForm();

		// Close button
		switch (event->data.ctlSelect.controlID)
		{
			// OK (yes, the name is not very well chosen!)
			case ID_BUTN_CLOS:
			WinDrawChars("Hello World !", 13, 20, 20);
			break;
		}
		break;

		//----------------------------------------
		// Pen event
		//----------------------------------------
		case penDownEvent:
		form = FrmGetActiveForm();

		// Get form bounds
		FrmGetFormBounds(form, &rect);

		// Convert position coordinate to screen values
		x = event -> screenX;
		y = event -> screenY;
		WinWindowToDisplayPt(&x, &y);

		// Check for a tap outside the form
		if (!RctPtInRectangle(x, y, &rect))
		{
			// Quit DA
			done = true;
		}
		break;

		//----------------------------------------
		// Quit
		//----------------------------------------
		case appStopEvent:
		// Transfer event to current app
		EvtAddEventToQueue(event);

		// Quit DA
		done = true;
		break;
	}

	return done;
}

//----------------------------------------------------
// Run DA: initialize and show UI
//----------------------------------------------------
static void StartApplication(FormPtr * form)
{
	// Load the form resource.
	*form = FrmInitForm(ID_FORM_MAIN);

	// Check if OK
	if (*form)
	{
		// Set as active form
		FrmSetActiveForm(*form);

		// Show form
		FrmDrawForm(*form);
	}
}

//----------------------------------------------------
// Manage event loop for the DA
//----------------------------------------------------
static void EventLoop(void)
{
	EventType event;
	Boolean   done = false;

	do
	{
		// Wait next event
		EvtGetEvent(&event, evtWaitForever);

		// System event processing & system form handler
		if (!SysHandleEvent(&event) && !FrmHandleEvent(FrmGetActiveForm(), &event))
		{
			// Local form handler
			done = MainFormHandleEvent(&event);
		}
	}
	while (!done);
}

//----------------------------------------------------
// Clean-up before quitting
//----------------------------------------------------
static void StopApplication(FormPtr form)
{
	// Remove form from screen
	FrmEraseForm(form);

	// Erase form data from memory
	FrmDeleteForm(form);
}

//----------------------------------------------------
// DA entry point
//----------------------------------------------------
void DAMain(void)
{
	FormPtr form = NULL;

	// Start application
	StartApplication(&form);

	// Test if OK
	if (form != NULL)
	{
		// Run
		EventLoop();

		// Stop
		StopApplication(form);
	}
}
